import type { ArgConfig, ArgsParserInterface, ParsedArgumentsCollectionInterface } from "./types";
/**
 * A collection of parsed arguments.
 *
 * @category Classes
 */
export declare class ParsedArgumentsCollection implements ParsedArgumentsCollectionInterface {
    private readonly args;
    /**
     * Adds a new argument to the collection.
     * @param name - The name of the argument.
     * @param value - The value of the argument.
     * @returns The instance of ParsedArgumentsCollection for chaining.
     */
    add(name: string, value: unknown): this;
    /**
     * Retrieves an argument value.
     * @param name - The name of the argument.
     * @returns The value of the argument, or undefined if not found.
     */
    get<T = unknown>(name: string): T;
    /**
     * Checks if an argument exists in the collection.
     * @param name - The name of the argument.
     * @returns True if the argument exists, otherwise false.
     */
    has(name: string): boolean;
    /**
     * Returns all arguments as a record.
     */
    get all(): Record<string, unknown>;
    /**
     * Formats the argument name by removing leading dashes.
     * @param name - The argument name.
     * @returns The formatted name.
     */
    private formatName;
}
/**
 * Parser for command-line arguments.
 *
 * @category Classes
 */
export declare class ArgsParser implements ArgsParserInterface {
    private readonly argsMap;
    private readonly aliasMap;
    private readonly usedArgs;
    /**
     * Constructs an instance of ArgsParser with the provided argument configurations.
     * @param args - An array of argument configurations.
     */
    constructor(args?: ArgConfig[]);
    /**
     * Adds a new argument configuration to the parser.
     * @param config - The argument configuration.
     * @returns The instance of ArgsParser for chaining.
     */
    addArgument(config: ArgConfig): ArgsParser;
    /**
     * Parses the given argument string and returns a collection of parsed arguments.
     * @param argsString - The argument string.
     * @returns A ParsedArgumentsCollection containing the parsed arguments.
     * @throws ArgumentValueError if a required argument is missing or an argument value is invalid.
     */
    parse(argsString: string): ParsedArgumentsCollection;
    /**
     * Formats an argument configuration.
     * @param config - The argument configuration.
     * @returns A formatted argument configuration.
     */
    private formatArgConfig;
    /**
     * Formats the argument name and alias for error messages.
     * @param argConfig - The argument configuration.
     * @returns A formatted string with the argument name and alias.
     */
    private formatNameWithAlias;
    /**
     * Checks the validity of an argument configuration.
     * @param config - The argument configuration.
     * @throws AddArgumentError if the configuration is invalid.
     */
    private checkArgumentConfig;
    /**
     * Retrieves the argument configuration for a given key.
     * @param key - The argument key.
     * @returns The corresponding FormattedArgConfig.
     * @throws ArgumentNameError if the argument key is unknown.
     */
    private getArgConfig;
}
