import { AddArgumentError, ArgumentNameError, ArgumentValueError } from "./exceptions";
import { castArgValue, parseArgsString } from "./utils";
/**
 * A collection of parsed arguments.
 *
 * @category Classes
 */
export class ParsedArgumentsCollection {
    constructor() {
        this.args = {};
    }
    /**
     * Adds a new argument to the collection.
     * @param name - The name of the argument.
     * @param value - The value of the argument.
     * @returns The instance of ParsedArgumentsCollection for chaining.
     */
    add(name, value) {
        this.args[this.formatName(name)] = value;
        return this;
    }
    /**
     * Retrieves an argument value.
     * @param name - The name of the argument.
     * @returns The value of the argument, or undefined if not found.
     */
    get(name) {
        var _a;
        return ((_a = this.args[this.formatName(name)]) !== null && _a !== void 0 ? _a : undefined);
    }
    /**
     * Checks if an argument exists in the collection.
     * @param name - The name of the argument.
     * @returns True if the argument exists, otherwise false.
     */
    has(name) {
        return this.args[this.formatName(name)] !== undefined;
    }
    /**
     * Returns all arguments as a record.
     */
    get all() {
        return Object.assign({}, this.args);
    }
    /**
     * Formats the argument name by removing leading dashes.
     * @param name - The argument name.
     * @returns The formatted name.
     */
    formatName(name) {
        return name.replace(/^--/, '');
    }
}
/**
 * Parser for command-line arguments.
 *
 * @category Classes
 */
export class ArgsParser {
    /**
     * Constructs an instance of ArgsParser with the provided argument configurations.
     * @param args - An array of argument configurations.
     */
    constructor(args = []) {
        this.argsMap = new Map();
        this.aliasMap = new Map();
        this.usedArgs = new Set();
        for (const arg of args) {
            this.addArgument(arg);
        }
    }
    /**
     * Adds a new argument configuration to the parser.
     * @param config - The argument configuration.
     * @returns The instance of ArgsParser for chaining.
     */
    addArgument(config) {
        this.checkArgumentConfig(config);
        this.argsMap.set(config.name, this.formatArgConfig(config));
        this.usedArgs.add(config.name);
        if (config.alias !== undefined) {
            this.aliasMap.set(config.alias, config.name);
            this.usedArgs.add(config.alias);
        }
        return this;
    }
    /**
     * Parses the given argument string and returns a collection of parsed arguments.
     * @param argsString - The argument string.
     * @returns A ParsedArgumentsCollection containing the parsed arguments.
     * @throws ArgumentValueError if a required argument is missing or an argument value is invalid.
     */
    parse(argsString) {
        const required = new Set([...this.argsMap.values()].filter((x) => x.required));
        const notEmpty = new Set([...this.argsMap.values()].filter((x) => x.notEmpty));
        const hasDefault = new Set([...this.argsMap.values()].filter((x) => x.default !== undefined));
        const parsedArgs = parseArgsString(argsString);
        const result = new ParsedArgumentsCollection();
        for (const [key, value] of Object.entries(parsedArgs)) {
            const argConfig = this.getArgConfig(key);
            if (notEmpty.has(argConfig) && value === '') {
                throw new ArgumentValueError(`Argument ${this.formatNameWithAlias(argConfig)} value cannot be empty.`);
            }
            if (argConfig.allowedValues !== undefined && !argConfig.allowedValues.includes(value)) {
                throw new ArgumentValueError(`Argument ${this.formatNameWithAlias(argConfig)} value must be one of ${argConfig.allowedValues.join(', ')}.`);
            }
            if (argConfig.validator !== undefined && !argConfig.validator(value)) {
                throw new ArgumentValueError(`Argument ${this.formatNameWithAlias(argConfig)} value is invalid.`);
            }
            required.delete(argConfig);
            hasDefault.delete(argConfig);
            result.add(argConfig.name, castArgValue(value, argConfig.type, argConfig.multiple, argConfig.default));
        }
        for (const argConfig of hasDefault) {
            result.add(argConfig.name, argConfig.default);
            required.delete(argConfig);
        }
        if (required.size > 0) {
            throw new ArgumentValueError(`Required arguments not provided: ${[...required].map((x) => x.name).join(', ')}`);
        }
        return result;
    }
    /**
     * Formats an argument configuration.
     * @param config - The argument configuration.
     * @returns A formatted argument configuration.
     */
    formatArgConfig(config) {
        var _a, _b, _c;
        return Object.assign(Object.assign({}, config), { required: (_a = config.required) !== null && _a !== void 0 ? _a : false, notEmpty: (_b = config.notEmpty) !== null && _b !== void 0 ? _b : false, multiple: (_c = config.multiple) !== null && _c !== void 0 ? _c : false });
    }
    /**
     * Formats the argument name and alias for error messages.
     * @param argConfig - The argument configuration.
     * @returns A formatted string with the argument name and alias.
     */
    formatNameWithAlias(argConfig) {
        return `${argConfig.name} ${argConfig.alias ? `(${argConfig.alias})` : ''}`;
    }
    /**
     * Checks the validity of an argument configuration.
     * @param config - The argument configuration.
     * @throws AddArgumentError if the configuration is invalid.
     */
    checkArgumentConfig(config) {
        if (!config.name.startsWith('--')) {
            throw new AddArgumentError(`Argument name must start with '--' ("${config.name}" given).`);
        }
        if (config.alias !== undefined && !config.alias.startsWith('-')) {
            throw new AddArgumentError(`Argument alias must start with '-' ("${config.alias}" given).`);
        }
        if (this.usedArgs.has(config.name)) {
            throw new AddArgumentError(`Argument ${config.name} already exists.`);
        }
        if (config.alias !== undefined && this.usedArgs.has(config.alias)) {
            throw new AddArgumentError(`Argument ${config.alias} already exists.`);
        }
    }
    /**
     * Retrieves the argument configuration for a given key.
     * @param key - The argument key.
     * @returns The corresponding FormattedArgConfig.
     * @throws ArgumentNameError if the argument key is unknown.
     */
    getArgConfig(key) {
        var _a;
        const config = this.argsMap.get((_a = this.aliasMap.get(key)) !== null && _a !== void 0 ? _a : key);
        if (config === undefined) {
            throw new ArgumentNameError(`Unknown argument ${key}.`);
        }
        return config;
    }
}
//# sourceMappingURL=classes.js.map