"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.castArgValue = castArgValue;
exports.parseArgsString = parseArgsString;
/**
 * Cast a string value to the given type.
 *
 * @param value The string value to cast.
 * @param type The type to cast the value to.
 * @param multiple Whether the value is a multiple value.
 * @param defaultValue The default value if the value is empty.
 * @returns The cast value.
 *
 * @category Utils
 */
function castArgValue(value, type, multiple, defaultValue) {
    if (multiple) {
        var result = value.split(' ').filter(function (x) { return x !== ''; }).map(function (x) { return castArgValue(x, type, false); });
        return result.length > 0 ? result : (defaultValue !== null && defaultValue !== void 0 ? defaultValue : []);
    }
    var _value = value !== '' ? value : (defaultValue !== null && defaultValue !== void 0 ? defaultValue : value);
    if (_value === undefined) {
        return _value;
    }
    switch (type) {
        case 'string': return _value;
        case 'number': return Number(_value);
        case 'boolean': return _value !== 'false' && _value !== '0';
    }
}
/**
 * Parse a string arguments to a plain object.
 *
 * @param argsString The string arguments.
 * @returns The parsed plain object.
 *
 * @category Utils
 */
function parseArgsString(argsString) {
    return Object.fromEntries(parseArgsArray(splitArgsString(formatArgsString(argsString))));
}
/**
 * Parse an array of string arguments to an array of tuples.
 *
 * @param args The array of string arguments.
 * @returns The parsed array of tuples.
 *
 * @category Utils
 */
function parseArgsArray(args) {
    return args
        .map(function (x) { return x.split(/ (.+)/).slice(0, 2); })
        .map(function (x) { return x.length === 1 ? [x[0], ''] : x; })
        .filter(function (x) { return x[0] !== ''; });
}
/**
 * Split an arguments string to an array of strings.
 *
 * @param argsString The string arguments.
 * @returns The split array of strings.
 *
 * @category Utils
 */
function splitArgsString(argsString) {
    return argsString.split(' -').map(function (x, i) { return i === 0 ? x : "-".concat(x); });
}
/**
 * Format an arguments string.
 *
 * @param argsString The string arguments.
 * @returns The formatted string.
 *
 * @category Utils
 */
function formatArgsString(argsString) {
    return argsString
        .trim()
        .replace(/\s+/g, ' ')
        .replace(/^(-[A-Za-z0-9]{2,})/, function (_, p) { return ' ' + formatGluedArgsString(p); })
        .replace(/ (-[A-Za-z0-9]{2,})/, function (_, p) { return ' ' + formatGluedArgsString(p); })
        .replace(/\s+/g, ' ')
        .trim();
}
/**
 * Format a glued arguments string.
 *
 * @param gluedArgsString The glued arguments string.
 * @returns The formatted string.
 *
 * @category Utils
 */
function formatGluedArgsString(gluedArgsString) {
    return gluedArgsString.slice(1).split('').map(function (x) { return "-".concat(x); }).join(' ');
}
//# sourceMappingURL=utils.js.map